﻿using ProjectPlane.Drawnings;
using ProjectPlane.MovementStrategy;

namespace ProjectPlane;

public partial class FormContainer : Form
{

    private DrawningShip? _drawShip;

    private AbstractStrategy? _strategy;

    public FormContainer()
    {
        InitializeComponent();
        _strategy = null;
    }

    private void Draw()
    {
        if (_drawShip == null)
        {
            return;
        }

        Bitmap bmp = new(pictureBoxCont.Width, pictureBoxCont.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _drawShip.DrawTransport(gr);
        pictureBoxCont.Image = bmp;
    }

    private void CreateObject(string type)
    {
        Random random = new Random();
        switch (type)
        {
            case nameof(DrawningShip):
                _drawShip = new DrawningShip(random.Next(100, 300), random.Next(1000, 3000),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)));
                break;
            case nameof(DrawCont):
                _drawShip = new DrawCont(random.Next(100, 300), random.Next(1000, 3000),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
                    Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)));
                break;
            default:
                return;
        }
        _drawShip.SetPictureSize(pictureBoxCont.Width, pictureBoxCont.Height);
        _drawShip.SetPosition(random.Next(10, 100), random.Next(10, 100));
        _strategy = null;
        comboBoxStrategy.Enabled = true;
        Draw();
    }

    private void ButtonCreate_Click(object sender, EventArgs e) => CreateObject(nameof(DrawCont));

    private void ButtonCreateShip_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningShip));

    private void ButtonMove_Click(object sender, EventArgs e)
    {
        if (_drawShip == null)
        {
            return;
        }

        string name = ((Button)sender)?.Name ?? string.Empty;
        bool result = false;
        switch (name)
        {
            case "buttonUp":
                result = _drawShip.MoveTransport(DirectionType.Up);
                break;
            case "buttonDown":
                result = _drawShip.MoveTransport(DirectionType.Down);
                break;
            case "buttonLeft":
                result = _drawShip.MoveTransport(DirectionType.Left);
                break;
            case "buttonRight":
                result = _drawShip.MoveTransport(DirectionType.Right);
                break;
        }

        if (result)
        {
            Draw();
        }
    }

    private void ButtonStrategyStep_Click(object sender, EventArgs e)
    {
        if (_drawShip == null)
        {
            return;
        }

        if (comboBoxStrategy.Enabled)
        {
            _strategy = comboBoxStrategy.SelectedIndex switch
            {
                0 => new MoveToCenter(),
                1 => new MoveToBorder(),
                _ => null,
            };
            if (_strategy == null)
            {
                return;
            }
            _strategy.SetData(new MoveableShip(_drawShip), pictureBoxCont.Width, pictureBoxCont.Height);
        }

        if (_strategy == null)
        {
            return;
        }

        comboBoxStrategy.Enabled = false;
        _strategy.MakeStep();
        Draw();

        if (_strategy.GetStatus() == StrategyStatus.Finish)
        {
            comboBoxStrategy.Enabled = true;
            _strategy = null;
        }

    }

}